/*
 * PM - 2016
 * lab0.c
 */

#define F_CPU 16000000
#include "ST7735.h"
#include <avr/io.h>
#include <util/delay.h>

#define DELAY_SECVENTA		250
#define NUM_SECVENTE		6

#define MORSE_LUNG		3
#define MORSE_SCURT		1

#define NUM_NOTE		8
#define DURATA_NOTA		100

float frecventa_nota[8] = {
	261.63, 293.66, 329.63, 349.23, 392.0, 440.0, 493.88, 523.25
};

/* Aprinde led-ul PC0 la apasarea butonului PB0. */
void exemplu(void) {
	/* Setez pinul PB0 ca intrare. */
	DDRB &= ~(1 << PB0);

	/* Activez rezistenta de pull-up pentru pinul PB0. */
	PORTB |= (1 << PB0);

	/* Setez pinul PC0 ca iesire. */
	DDRC |= (1 << PC0);

	/* Sting LED-ul. */
	PORTC &= ~(1 << PC0);

	while (1) {
		/* Daca butonul este apasat. */
		if ((PINB & (1 << PB0)) == 0)
			/* Aprind LED-ul. */
			PORTC |= (1 << PC0);
		else
			/* Sting LED-ul. */
			PORTC &= ~(1 << PC0);
	}
}

/*
 * Aprinde LED-urile legate la portul C in functie de secventele alese.
 * Secventa va fi selectata la apasarea butonului PB0 sau PB1.
 */
void task1(void) {
	/* Numar secventa curenta. */
	int secventa = 0;
	int i = 0;

	/* Setez pinii PB0 si PB1 ca pini de intrare. */
	DDRB &= ~((1 << PB0) | (1 << PB1));

	/* Activez rezistenta de pull-up pentru pinii PB0 si PB1. */
	PORTB |= (1 << PB0) | (1 << PB1);

	/* Setez toti pinii portului C ca pini de iesire. */
	DDRC = 0xFF;

	/* Sting toate LED-urile*/
	PORTC = 0x00;

	while(1) {
		/* Verific daca PB0 este apasat. */
		if ((PINB & (1 << PB0)) == 0) {
			if (secventa < NUM_SECVENTE - 1)
				++secventa;

			/* Sting LED-urile pentru a pregati noua secventa. */
			PORTC = 0x00;
			i = 0;
		} else if ((PINB & (1 << PB1)) == 0) {
			if (secventa > 0)
				--secventa;

			/* Sting LED-urile pentru a pregati noua secventa. */
			PORTC = 0x00;
			i = 0;
		}

		/* Iterez prin secvente in functie de cea selectata. */
		switch (secventa) {
			/* Simplu 1*/
			case 0:
				i %= 8;

				PORTC = (1 << i);
				break;

				/* Simplu 2 */
			case 1:
				i %= 8;

				if (i == 0)
					PORTC = 0x00;

				PORTC |= (1 << i);
				break;

				/* Simplu 3 */
			case 2:
				i %= 8;

				PORTC = ((1 << i) | (1<< (7 - i)));
				break;

				/* Avansat 1 */
			case 3:
				i %= 15;

				if (i < 8)
					PORTC = (1 << i);
				else
					PORTC = (1 << (14 - i));
				break;

				/* Avansat 2 */
			case 4:
				i %= 16;

				if (i == 0)
					PORTC = 0x00;

				if (i < 8)
					PORTC |= (1 << i);
				else
					PORTC &= ~(1 << (i - 8));
				break;

				/* Avansat 3 */
			case 5:
				i %= 3;

				PORTC = (PORTC << 1);

				if(i == 0)
					PORTC |= 0x01;
				break;
		}

		i++;
		_delay_ms(DELAY_SECVENTA);
	}
}

/* Genereaza sunetul pentru tipul de ton folosit: lung sau scurt. */
void speaker_morse(int tip_morse) {
	/*
	 * Durata, in milisecunde, in functie de tipul de ton folosit: 50 ms
	 * pentru un ton scurt, 150 ms pentru un ton lung.
	 */
	int durata = 50 * tip_morse;

	int i;

	/* Alternam starea pinului PD5 la intervale de timp foarte mici. */
	for (i = 0; i < durata; ++i) {
		PORTD ^= (1 << PD5);

		_delay_ms(1);
	}
}

/*
 * Genereaza tonuri pentru codul Mores folosind buzzer-ul. Tonul generat este
 * selectat folosind butoanele PB0 si PB1.
 */
void task2(void) {
	/* Setez pinii PB0 si PB1 ca intrare. */
	DDRB &= ~((1 << PB0) | (1 << PB1));

	/* Activez rezistenta de pull-up pentru pinii PB0 si PB1. */
	PORTB |= (1 << PB0) | (1 << PB1);

	/* Setez pinul PD5 ca iesire. */
	DDRD |= (1 << PD5);

	/* Opresc buzzer-ul. */
	PORTD &= ~(1 << PD5);

	while (1) {
		if ((PINB & (1 << PB0)) == 0)
			speaker_morse(MORSE_SCURT);
		else if ((PINB & (1 << PB1)) == 0)
			speaker_morse(MORSE_LUNG);

		PORTD &= ~(1 << PD5);
		_delay_ms(50);
	}
}

/* Genereaza o nota folosind buzzer-ul. */
void buzzer_reda(int nota) {
	float perioada_nota = 100000 / frecventa_nota[nota];
	int i, j;

	for (i = 0; i < DURATA_NOTA; ++i) {
		PORTD |= (1 << PD5);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0; j < perioada_nota / 2; ++j)
			_delay_us(1);

		PORTD &= ~(1 << PD5);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0 ; j < perioada_nota / 2; ++j)
			_delay_us(1);
	}
}

/* Aprinde LED-ul corespunzator notei alese. */
void led_aprinde(int nota) {
	PORTC = (1 << nota);
}

/* Stinge LED-urile. */
void led_stinge(void) {
	PORTC = 0x00;
}

/*
 * Genereaza note folosind buzzer-ul. Nota generata este aleasa folosind
 * butoanele PB0 si PB1.
 */
void task3(void) {
	int nota_curenta = -1;

	/* Setez pinii PB0 si PB1 ca pini de intrare. */
	DDRB &= ~((1 << PB0) | (1 << PB1));

	/* Activez rezistenta de pull-up pentru pinii PB0 si PB1. */
	PORTB |= (1 << PB0) | (1 << PB1);

	/* Setez toti pinii portului C ca pini de iesire. */
	DDRC = 0xFF;

	/* Sting toate LED-urile*/
	PORTC = 0x00;

	/* Setez pinul PD5 ca iesire. */
	DDRD |= (1 << PD5);

	/* Opresc buzzer-ul. */
	PORTD &= ~(1 << PD5);

	while (1) {
		if ((PINB & (1 << PB0)) == 0) {
			if (nota_curenta < NUM_NOTE - 1)
				++nota_curenta;
		}
		else if ((PINB & (1 << PB1)) == 0) {
			if (nota_curenta > -1)
				--nota_curenta;
		}

		if (nota_curenta != -1) {
			/* Aprindem LED-ul corespunzator. */
			led_aprinde(nota_curenta);

			/* Redam nota. */
			buzzer_reda(nota_curenta);
		} else {
			/* Stingem LED-urile. */
			led_stinge();
		}

		_delay_ms(50);
	}
}
uint8_t character[10][10] =
						{0,0,1,1,1,0,0,0,0,0,
						 0,0,1,1,1,0,1,0,0,0,
						 0,0,1,1,0,0,1,0,0,0,
						 0,1,1,1,1,1,1,0,0,0,
						 1,0,1,1,1,0,0,0,0,0,
						 0,1,1,1,1,0,0,0,0,0,
						 0,0,1,1,1,1,0,0,0,0,
						 0,0,1,0,0,1,0,0,0,0,
						 0,1,0,0,0,0,1,0,0,0,
						 1,1,0,0,0,0,1,1,0,0};

uint8_t characterPunching[10][10] =
						{0,0,0,0,1,1,1,0,0,0,
						 0,0,0,0,1,1,1,0,0,0,
						 0,0,0,0,1,1,0,0,0,0,
						 0,0,1,1,1,1,1,1,1,1,
						 0,0,1,1,1,1,0,0,0,0,
						 0,0,0,1,1,1,0,0,0,0,
						 0,0,1,1,1,1,1,0,0,0,
						 0,0,1,0,0,0,1,1,0,0,
						 0,1,0,0,0,0,1,0,0,0,
						 1,1,0,0,0,0,1,1,0,0};

void drawCharacter(int sy, int sx, uint8_t a[10][10], uint16_t color){
	int i,j;

	for(i = 0; i < 10; i++){
		for(j = 0; j < 10; j++){
			if(a[i][j]){
				TFTPixel(sy+2*i,sx+2*j,color);
				TFTPixel(sy+2*i + 1,sx+2*j,color);
				TFTPixel(sy+2*i,sx+2*j + 1,color); 
				TFTPixel(sy+2*i + 1,sx+2*j + 1,color);
			}
		}	
	}
}
void drawUke(int sy, int sx, uint8_t a[10][10], uint16_t color){
	int i,j;

	for(i = 0; i < 10; i++){
		for(j = 0; j < 10; j++){
			if(a[i][j]){
				TFTPixel(sy+2*i,sx+2*(10-j),color);
				TFTPixel(sy+2*i + 1,sx+2*(10-j),color);
				TFTPixel(sy+2*i,sx+2*(10-j) + 1,color); 
				TFTPixel(sy+2*i + 1,sx+2*(10-j) + 1,color);
			}
		}	
	}
}
void clscr(uint16_t color){
	int i,j;
	for(i = 0;i < 128; i++)
		for (j = 0; j < 160;j++)
				TFTPixel(i,	j, color);
}
int main(void) {

	uint16_t cury, curx, lasty,lastx;
	cury = curx = lasty = lastx = 0;
	cury = lasty = 50;
	int i = 0,j, increasing = 1;
	TFTInit();
	TFTSetWindow(0,0,128,160);
	DDRD &= ~15;
	PORTD |= 15;
	char punchFrames = 0;
	clscr(BLUE);
	for(i = 70; i < 128; i++)
		for (j = 0; j < 160;j++)
				TFTPixel(i,	j, GREEN);
	while(1){
		lastx = curx;
		lasty = cury;
		
		
		if ((PIND & (1 << PD0)) == 0)
			curx--;
		if((PIND & (1 << PD1)) == 0)
			curx++;

		if(punchFrames == 0 && ((PIND & (1 << PD3)) == 0))
			punchFrames = 3;

		drawUke(50,100,character, RED);//sterg ecranul doar unde am desenat
		if(punchFrames == 3){
			drawCharacter(lasty,lastx,character, BLUE);//sterg ecranul doar unde am desenat
			drawCharacter(cury, curx, characterPunching, YELLOW);
		}
		else if(punchFrames > 1){
			drawCharacter(lasty,lastx,characterPunching, BLUE);//sterg ecranul doar unde am desenat
			drawCharacter(cury, curx, characterPunching, YELLOW);
		}else if (punchFrames == 1){
			drawCharacter(lasty,lastx,characterPunching, BLUE);//sterg ecranul doar unde am desenat
			drawCharacter(cury,curx,character, YELLOW);//sterg ecranul doar unde am desenat

		}else {
			drawCharacter(lasty,lastx,character, BLUE);//sterg ecranul doar unde am desenat
			drawCharacter(cury,curx,character, YELLOW);//sterg ecranul doar unde am desenat
		}
		

		if(punchFrames > 0 )
			punchFrames--;
	}
	return 0;

	/* Setez pinii PB0 si PB1 ca pini de intrare. */
	DDRB &= ~((1 << PB0) | (1 << PB1));

	/* Activez rezistenta de pull-up pentru pinii PB0 si PB1. */
	PORTB |= (1 << PB0) | (1 << PB1);

	/* Setez toti pinii portului C ca pini de iesire. */
	DDRC = 0xFF;

	/* Sting toate LED-urile*/
	PORTC = 0x00;
}